package org.kantega.jexmec.logging;

/*
 * Copyright 2011 Kantega AS
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import org.kantega.jexmec.ClassLoaderProvider;
import org.kantega.jexmec.PluginLoader;
import org.kantega.jexmec.PluginManager;
import org.kantega.jexmec.PluginManagerListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Collection;

/**
 * A plugin manager listener logging all events to an SLF4J logger
 */
public class LoggingPluginManagerListener<P> extends PluginManagerListener<P> {
    private final Logger logger;

    /**
     * Creates a LoggingPluginManagerListener logging to the given Logger.
     *
     * @param logger the Logger to log events to
     */
    public LoggingPluginManagerListener(Logger logger) {
        this.logger = logger;
    }

    /**
     * Creates a LoggingPluginManagerListener logging to the default Logger,
     * retrieved by calling <code>LoggerFactory.getLogger(LoggingPluginManagerListener.class)</code>
     */
    public LoggingPluginManagerListener() {
        this(LoggerFactory.getLogger(LoggingPluginManagerListener.class));
    }

    @Override
    public void beforePluginManagerStarted(PluginManager<P> pluginManager) {
        if (logger.isInfoEnabled())
            logger.info("Plugin manager is about to start: " + pluginManager);
    }

    @Override
    public void afterPluginManagerStarted(PluginManager pluginManager) {
        if (logger.isInfoEnabled())
            logger.info("Plugin manager has started: " + pluginManager);
    }

    @Override
    public void beforePluginManagerStopped(PluginManager<P> pluginManager) {
        if (logger.isInfoEnabled())
            logger.info("Plugin manager is about to stop: " + pluginManager);
    }


    @Override
    public void afterPluginManagerStopped(PluginManager pluginManager) {
        if (logger.isInfoEnabled())
            logger.info("Plugin manager has stopped: " + pluginManager);
    }


    @Override
    public void beforeClassLoaderAdded(PluginManager<P> pluginManager, ClassLoaderProvider classLoaderProvider, ClassLoader classLoader) {
        if (logger.isInfoEnabled())
            logger.info("Class loader is about to be added: " + classLoader + " for provider " + classLoaderProvider + " for plugin manager " + pluginManager);
    }


    @Override
    public void afterClassLoaderAdded(PluginManager<P> pluginManager, ClassLoaderProvider classLoaderProvider, ClassLoader classLoader) {
        if (logger.isInfoEnabled())
            logger.info("Class loader has been added: " + classLoader + " for provider " + classLoaderProvider + " for plugin manager " + pluginManager);
    }


    @Override
    public void beforeClassLoaderRemoved(PluginManager<P> pluginManager, ClassLoaderProvider classLoaderProvider, ClassLoader classLoader) {
        if (logger.isInfoEnabled())
            logger.info("Class loader is about to be removed: " + classLoader + " for provider " + classLoaderProvider + " for plugin manager " + pluginManager);
    }

    @Override
    public void afterClassLoaderRemoved(PluginManager<P> pluginManager, ClassLoaderProvider classLoaderProvider, ClassLoader classLoader) {
        if (logger.isInfoEnabled())
            logger.info("Class loader has been removed: " + classLoader + " for provider " + classLoaderProvider + " for plugin manager " + pluginManager);
    }

    @Override
    public void beforeActivation(PluginManager<P> pluginManager, ClassLoaderProvider classLoaderProvider, ClassLoader classLoader, PluginLoader<P> pluginLoader, Collection<P> plugins) {
        if (logger.isInfoEnabled())
            logger.info("Plugins " + plugins + " loaded from class loader " + classLoader + " by plugin loader " + pluginLoader + " are about to be activated for provider " + classLoaderProvider + " for manager " + pluginManager);
    }


    @Override
    public void afterActivation(PluginManager<P> pluginManager, ClassLoaderProvider classLoaderProvider, ClassLoader classLoader, PluginLoader<P> pluginLoader, Collection<P> plugins) {
        if (logger.isInfoEnabled())
            logger.info("Plugins " + plugins + " loaded from class loader " + classLoader + " by plugin loader " + pluginLoader + " have been activated for provider " + classLoaderProvider + " for manager " + pluginManager);
    }

    @Override
    public void beforePassivation(PluginManager<P> pluginManager, ClassLoaderProvider classLoaderProvider, ClassLoader classLoader, PluginLoader<P> pluginLoader, Collection<P> plugins) {
        if (logger.isInfoEnabled())
            logger.info("Plugins " + plugins + " loaded from class loader " + classLoader + " by plugin loader " + pluginLoader + " are about to be passivated for provider " + classLoaderProvider + " for manager " + pluginManager);
    }


    @Override
    public void afterPassivation(PluginManager<P> pluginManager, ClassLoaderProvider classLoaderProvider, ClassLoader classLoader, PluginLoader<P> pluginLoader, Collection<P> plugins) {
        if (logger.isInfoEnabled())
            logger.info("Plugins " + plugins + " loaded from class loader " + classLoader + " by plugin loader " + pluginLoader + " have been passivated for provider " + classLoaderProvider + " for manager " + pluginManager);
    }

    @Override
    public void pluginLoadingFailedWithException(PluginManager<P> pluginManager, ClassLoaderProvider classLoaderProvider, ClassLoader classLoader, PluginLoader<P> pluginLoader, Throwable exception) {
        if (logger.isErrorEnabled())
            logger.error("Plugin loader " + pluginLoader + " threw " + exception.getClass().getSimpleName() + " loading plugins from class loader " + classLoader + " provided by " + classLoaderProvider + " managed by " + pluginManager, exception);
    }


    @Override
    public void pluginsUpdated(Collection<P> plugins) {
        if (logger.isInfoEnabled())
            logger.info("Plugins list updated to: " + plugins);
    }

}
